module TimeRange
    exposing
        ( TimeRange
        , fromStartAndEnd
        , start
        , end
        , isContainedWithin
        , overlaps
        , difference
        )

{-| A library for representing time ranges in an elegant manner.

# Definition
@docs TimeRange, fromStartAndEnd

# Accessors
@docs start, end

# Common Helpers
@docs difference, isContainedWithin, overlaps
-}

import Time exposing (Time)


{-| Representation of a time range.
-}
type TimeRange
    = TimeRange
        { start : Time
        , end : Time
        }


{-| Construct a time range from start and end times.
-}
fromStartAndEnd : Time -> Time -> TimeRange
fromStartAndEnd start end =
    TimeRange
        { start = start
        , end = end
        }


{-| Get the start time for the range.
-}
start : TimeRange -> Time
start (TimeRange { start, end }) =
    start


{-| Get the end time for the range.
-}
end : TimeRange -> Time
end (TimeRange { start, end }) =
    end


{-| Allows you to find out if a given `Time` is contained within a `TimeRange`.
-}
isContainedWithin : Time -> TimeRange -> Bool
isContainedWithin time timeRange =
    start timeRange < time && time <= end timeRange


{-| Allows you to find out if two `TimeRange`s overlap.
-}
overlaps : TimeRange -> TimeRange -> Bool
overlaps timeRange1 timeRange2 =
    isContainedWithin (start timeRange2) timeRange1
        || isContainedWithin (end timeRange1) timeRange2


{-| Get the `Time` difference between the start and end time in the time range.
-}
difference : TimeRange -> Time
difference timeRange =
    abs <| end timeRange - start timeRange
